import cherrypy, os
import rootapp.pages

from cherrypy.lib.filter.sessionfilter import RamStorage, generateSessionID
from cherrypy.lib import httptools

# import the epl interface
import eplapi
import eplglobals

import kidpages.isp.login
import kidpages.isp.index
import kidpages.isp.addsite
import kidpages.isp.addplan
import kidpages.isp.editplan
import kidpages.isp.editsite
import kidpages.isp.addreseller
import kidpages.isp.editreseller
import kidpages.isp.addsiteadv
import kidpages.isp.networksettings
import kidpages.isp.diskusage
import kidpages.isp.timezone
import kidpages.isp.edittimezone
import kidpages.isp.show_serveraction
import kidpages.isp.changeadmin
import kidpages.isp.changelicense
import kidpages.isp.displayLicense
import kidpages.isp.displayeula
import kidpages.common.uploadlogo
import kidpages.common.generallist
import kidpages.common.manageskins
import kidpages.common.previewskin
#import kidpages.common.liveactionlog # keepalive
import kidpages.isp.topbar
import kidpages.isp.footer

from lib.datamodel.datatypes import ipaddr
from lib.datamodel.controls import Column
from lib.datamodel.ispviews import SiteListEntryView, PlanListEntryView,ServiceListEntryView
from lib.common import parsealias

import lib.xmlutils
from lib.threads.keepalive import keepalive
from eplapi.licensing import EncryptDecrypt,licenselib,callhome
from SystemState import SystemState, EStateTransition
from cherrypy.lib.cptools import serveFile

class Page (rootapp.pages.ISPPage):

    def login (self, fromPage, login = '', errorMsg = ''):
        """ return the login screen for isp admin """

        cookie = cherrypy.response.simpleCookie

        if "epl_isp" in cookie:
            del cookie["epl_isp"]

        #storing the template instance in variable 'template'
        template = self.kidglobals (kidpages.isp.login)

        template.languageMap = self.initializedLanguageMap ()

        template.errorMsg = errorMsg
        template.fromPage = fromPage
        return self.kidserialize (template)

    def dologin (self, username, password):
        """ 
           dologin (self, username, password):
           this function is responsible for authenticating the isp administrator
           the signature of this function is actually governed by the sessionAuthenticateFilter

           this is the reason why we cannot accept the language parameter in the function signature
           this function is supposed to return an empty string if the login succeeds, otherwise
           a failure message, which will then be passed as the "errorMsg" parameter to the login()
           function. all this is done by the sessionauthenticate filter, and you are encouraged to
           read the file {site-packages}/cherrypy/lib/filter/sessionauthenticatefilter.py, in the
           beforeMain () function.
        """

        # typically, this function will create a role object from eplapi, and then store that in the 
        # session.

        # [note] Notice that the locale argument is not passed here, and we need to fetch it 
        # from the request object. This is because this 'dologin' function is special, and its 
        # signature is governed by the sessionAuthenticate filter.

        language = cherrypy.request.paramMap['language']

        try:
            # Create the User object. eplapi gives us this
            user = eplapi.roles ('isp') (username)
            user.authenticate (password)
            eplapi.context(user)
            self.pushtologinsstack (user)
            if not licenselib.isEULAPresent():
                cherrypy.session['trespasser']='/isp/index'
            elif licenselib.getMLK() == 0 or licenselib.isGrace():
                cherrypy.session['trespasser']='/isp/index'
            else:
                if licenselib.isTimeForCallHome() or licenselib.isExpired():
                   cherrypy.session['trespasser']='/isp/index'
            return ""
        except EplError, e:
            cookie = cherrypy.response.simpleCookie
            del cookie['epl_isp']
            return str (e)

    def index (self):
        #assigning template instance to variable 'template'
        if 'skipforcelicense' in cherrypy.session:
            template = self.kidglobals (kidpages.isp.index)
            return self.kidserialize (template)
            
        actionlog.clean()
        if not licenselib.isEULAPresent():
            # The EULA is not present until such time that the user agrees to the license agreement
            cherrypy.session['trespasser']='/isp/index'
            template = self.kidglobals (kidpages.isp.displayeula)
            return self.kidserialize (template)
	elif licenselib.getMLK() == 0 or licenselib.isGrace():
            # MLK is not present or the license is under grace period
            cherrypy.session['trespasser']='/isp/index'
            return self.renderChangeLic('index')
	else:
            #condition 'or' is not used because for expired licenses we want to show force registration page. 
            if licenselib.isTimeForCallHome() and licenselib.isExpired() != 2:
                (returnCode,returnMsg) = callhome.callMe()
                if returnCode == 2 and returnMsg != "SERVER_NOT_CONTACTABLE":
                    #Server not contactable should not show error message. It may be a temporary network problem
                    self.displayError(returnMsg)
                    cherrypy.session['trespasser']='/isp/index'
                    return self.renderChangeLic('index')

            if licenselib.isGrace() or licenselib.isExpired() == 2:
                cherrypy.session['trespasser']='/isp/index'
                return self.renderChangeLic('index')
            else:
                if 'trespasser' in cherrypy.session:
                   del cherrypy.session['trespasser']
                template = self.kidglobals (kidpages.isp.index)
                return self.kidserialize (template)

    index.exposed = True

    def navbar (self):
        return lib.xmlutils.xsltproc (self.StaticResource("xsl/isp.navbar.xsl"), self.StaticResource("xml/isp.navbar.xml"))

    navbar.exposed = True

    def dashboard (self):
        cherrypy.response.headerMap['Content-Type'] = 'text/xml'
        f = file("static/xml/isp.dashboard.xml")
        data = f.readlines()
        f.close()
        return data
        #return "/static/xml/isp.dashboard.xml"
    dashboard.exposed = True

    def main (self):
        return lib.xmlutils.xsltproc (self.StaticResource ("xsl/isp.main.xsl"), self.StaticResource ("xml/isp.dashboard.xml"))

    main.exposed = True

    def topbar (self):
        """ <appliancetop><uptime>1402738</uptime><server_name>lwpdev-vm194.india.ensim.com</server_name><hidden_services /><server_ip>10.76.4.194</server_ip><services><item>sendmail</item><item>mivamerchant</item><item>telnet</item><item>clamav</item><item>diskquota</item><item>spam_filter</item><item>mod_php</item><item>virthost</item><item>bind</item><item>scriptsmgr</item><item>mivamerchant5</item><item>vhbackup</item><item>anonftp</item><item>aliases</item><item>reseller</item><item>bandwidth</item><item>develenv</item><item>sqmail</item><item>analog</item><item>logrotate</item><item>majordomo</item><item>siteinfo</item><item>mod_perl</item><item>ssh</item><item>ssi</item><item>python</item><item>subdomain</item><item>tomcat4</item><item>frontpage</item><item>apache</item><item>cgi</item><item>vacation</item><item>openssl</item><item>files</item><item>weblogs</item><item>users</item><item>ipinfo</item><item>mysql</item><item>proftpd</item><item>mailscanner</item><item>imap</item><item>webalizer</item></services><domain_count>10</domain_count></appliancetop> """

        server = eplapi.server ()
        return lib.xmlutils.xsltproc_string (self.StaticResource ("xsl/isp.top.xsl"), server.topbar_xml())

    topbar.exposed = True

    def footer (self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.footer)
        return self.kidserialize (template)

    footer.exposed = True

    def listsites (self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.common.generallist)
        # Setting context to admin
        serverobj = eplapi.server()

        siteobjlist = serverobj.getAllSiteObjs()

        mysitelist = []
        for siteobj in siteobjlist:
            config = {}
            try:
                domain_info = siteobj.getDomainInfo()
            except:
                continue

            d_name = domain_info['domain']
            config['status'] = str(siteobj.state_enum[siteobj.state].key)
            config['site'] = domain_info['domain']
	    config['namebased'] = domain_info['namebased']
            #config['idn'] = ['site']
            if domain_info['namebased'] == '1':
                config['ipaddress'] = ipaddr(domain_info['nbaddrs'][0])
            else:
                config['ipaddress'] = ipaddr(domain_info['ipaddrs'][0])
            config['actions'] = 'none'
            config['admin_user'] = domain_info['admin_user']    # the name of the site admin without the @domain suffix

            mysitelist.append(config)
        # provide mysitelist to the generallist kid page.
        column_list = [
            Column (name = 'site',        title = _('Site'), width="", searchable = True, sortable = True),
            Column (name = 'status',      title = _('Status'), width="68", sortable = True),
            Column (name = 'namebased',      title = _('Type'),width="60", sortable = True),
            Column (name = 'ipaddress',   title = _('IP Address'), width="100", searchable = True, sortable = True),
            Column (name = 'actions',     title = _('Actions'), width="150")
        ]
        template.buttons = []
        template.list_title = _('Site List')
        template.column_list = column_list
        template.display_data = mysitelist
        template.entryview = SiteListEntryView
        template.display_view = "SITELIST"
            
        return self.kidserialize (template)

    listsites.exposed = True

    def dodeleteSite (self, name):
        try:
            obj = eplapi.server()
            obj.deleteSite(name)
        except:
            pass
        return self.listsites()

    dodeleteSite.exposed = True

    def listservices(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.common.generallist)
        # Setting context to admin
        serverobj = eplapi.server()
        
        serviceobjlist = [svc for svc in serverobj.getAllSvcComps() if svc.name not in eplapiCurrentUser().disabled_services]
        
        service_not_to_display = ['aspmgr', 'aliases', 'anonftp', 'vhbackup', 'bandwidth', 'cgi', 'develenv', 'diskquota', \
            'files', 'weblogs', 'ipinfo', 'logrotate', 'reseller', 'siteinfo', 'ssi', 'subdomain', 'users' ]

        # links to be added for all services which are configurable
        service_conf_links = {'apache': '/isp/services/apache/', \
                              'bind': '/isp/services/bind/displaysoasettings', \
                              'sendmail': '/isp/services/sendmail/index', \
                              'proftpd': '/isp/services/proftpd/index', \
                              'mailscanner': '/isp/services/mailscanner/index', \
                              'mysql': '/isp/services/mysql/', \
                              'spam_filter': '/isp/services/spam_filter/index' \
        }

        myservicelist = []
        for serviceobj in serviceobjlist:
            config = {}
            try:
                name = serviceobj.name
                if name in service_not_to_display:
                    continue
                config['name'] = name
                config['display_name'] = serviceobj.displayName
                config['version'] = serviceobj.rpmversion()
                config['status'] = serviceobj.status()
                config['link'] = service_conf_links.get(name, None)
                config['actions'] = None
            except:
                continue

            myservicelist.append(config)

        # To find the version of Clam Antivirus
        import rpm
        ts = rpm.ts()
        mi = ts.dbMatch ("name", "clamav")
        version_list = [ m['version'] for m in mi ]
        if len(version_list)!=0:
            clamav_version = version_list[0]
        else:
            clamav_version = ""
            
        myservicelist.append(dict(display_name=_('Virtual Hosting Tools'), version='', status=None, actions=None))
        myservicelist.append(dict(display_name=_('Virus Scanning (Clam AntiVirus)'), version=clamav_version, status=None, actions=None))

        # provide myservicelist to the generallist kid page.

        column_list = [
            Column (name = 'display_name',    title = _('Name'),   sortable = True,    searchable = True),
            Column (name = 'version',    title = _('Version'),   width='65'),
            Column (name = 'status',    title = _('Status'),   width='65', sortable = True),
            Column (name = 'actions', title = _('Actions'))
        ]

        template.column_list = column_list
        template.display_data = myservicelist
        template.entryview = ServiceListEntryView
        template.display_view = "SERVICELIST"
        template.list_title = _('Subscribed Services')

        template.helptext = {"title":_("Note"),
                          "text":[_("Stop any services that you are not utilizing on this server.")]}
        template.buttons = []
        return self.kidserialize (template)

    listservices.exposed = True
    
    def serviceaction(self, action, name):

        serviceobj = eplapi.services(name)

        if action == "start":
            serviceobj.start()
        elif action == "stop":
            serviceobj.stop()
        elif action == "restart":
            serviceobj.restart()
        
        return self.listservices()

    serviceaction.exposed = True

    def listplans(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals ( kidpages.common.generallist )
        serverobj = eplapi.server()

        plandict = serverobj.user.plans

        planlist = []
        for plan in plandict:
            myplandict = {}
            ### assigning namebased value to make url for addsite #####
            namebased = plandict[plan]._getConfig()['ipinfo']['namebased']
            
            myplandict['plan'] = plan
            myplandict['namebased'] = namebased
            myplandict['actions'] = 'none'
            planlist.append(myplandict)

        # provide mysitelist to the generallist kid page.
        column_list = [
            Column (name = 'plan',        title = _('Plan Name'), searchable = True, sortable = True),
            Column (name = 'actions',     title = _('Actions'))
        ]
        
        template.helptext = {"title":_("Plan List"),
                          "text":[_("A Service Plan is a template that defines the default resource assignments and service offerings to be applied when you create new sites. A Service Plan is not required to create a new domain, but it makes creating multiple domains faster and easier.")]}
        template.buttons = []
        template.list_title = _('Plan List')
        template.column_list = column_list
        template.display_data = planlist
        template.entryview = PlanListEntryView
        template.display_view = "PLANLIST"

        return self.kidserialize (template)

    listplans.exposed = True
    
    def addPlan(self,**args):
        #assigning tempalte instance to template variable
        template = self.kidglobals (kidpages.isp.addplan)
        config = {}
        
        if len(args)>0:
            ### when user change plan name ##
            plan_name = {'plan_name':args['plan_name']}
            plansignature = eplapiCurrentUser().plans[args['plan_name']].editPlanSignature(**plan_name)
        else:
            ## when user use add plan ###
            plansignature = eplapiCurrentUser().createPlanSignature('')
            
        for key, obj in plansignature.hashed_fields.items():
            config[key] = plansignature.hashed_fields[key]
        ### assigning Security level and namebased value ###
        apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
        telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
        ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
        if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                security_level = 'high'
        elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                security_level = 'medium'
        elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                security_level = 'low'
        namebased = str(int(bool(config['ipinfo_namebased'].field_dict['checked'])))
        ### assigning Alias default constraints ###
        config['aliases_aliases'].field_dict['constraints'] = '[\S]*$'
        ###  unit convirsion for bandwidth Threshold Byte to MB ###
        config['bandwidth_threshold'].field_dict['value'] = eplglobals.fromByte(config['bandwidth_threshold'].field_dict['value'],'MB')
        ### creating dict extra to make default selection in kid pages ###
        extra ={'siteinfo_security':security_level,'ipinfo_namebased':namebased}
        
        template.config = config
        template.extra = extra

        return self.kidserialize (template)
        
    addPlan.exposed = True

    def doaddPlan(self,**formargs):
        
        ### used convertStringToList() method to convert string with \n in list format for alias ##
        if 'aliases_aliases' in formargs:
            formargs['aliases_aliases'] = parsealias.convertStringToList(formargs['aliases_aliases'],'\n')
        ### create map for Security Level ###
        security_level = {'high':{'apache_jail':'1','telnet_jail':'1','ssh_jail':'1'},'medium':{'apache_jail':'0',\
            'telnet_jail':'1','ssh_jail':'1'},'low':{'apache_jail':'0','telnet_jail':'0','ssh_jail':'0'}}
        formargs.update(security_level[formargs['siteinfo_security']])
        ### unit convirsion for bandwidth Threshold MB to Byte ###
        if 'bandwidth_threshold' in formargs:
            formargs['bandwidth_threshold'] = eplglobals.toByte(formargs['bandwidth_threshold'],'MB')
        ### creating variable for either save with new plan or existing plan ###
        new_plan_name = formargs['new_plan']
        exist_plan_name = formargs['exist_plan']
        save_type = formargs['overWritePlan']
        EXISTPLAN = '1'
        NEWPLAN = '0'
        ### Removed unused key from formargs before sending to backend  ###
        del(formargs['exist_plan'],formargs['new_plan'],formargs['plan_name'],formargs['overWritePlan'],formargs['createplan'],formargs['planhidden'])
        ### Calling Backend for create Plan ###
        try:
            if save_type == NEWPLAN:
                plan = eplapiCurrentUser().createPlan(new_plan_name, **formargs)
            elif save_type == EXISTPLAN :
                plan = eplapiCurrentUser().plans[exist_plan_name].editPlan(**formargs)
        except EplError:
            error(_('Unable to create Plan'))
        return self.listplans()
        
    doaddPlan.exposed = True

    def editPlan(self,**args):
        #assigning template instance to template variable
        template = self.kidglobals (kidpages.isp.editplan)
        config = {}
        ### Creating plan dict from signatures### 
        plansignature = eplapiCurrentUser().plans[args['plan_name']].editPlanSignature(**args)
        
        for key, obj in plansignature.hashed_fields.items():
            config[key] = plansignature.hashed_fields[key]
        ####################### assigning Security level and namebased value ########################
        apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
        telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
        ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
        if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                security_level = 'high'
        elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                security_level = 'medium'
        elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                security_level = 'low'
        namebased = str(int(bool(config['ipinfo_namebased'].field_dict['checked'])))
        ########################### assigning Alias default constraints ##############################
        config['aliases_aliases'].field_dict['constraints'] = '[\S]*$'
        #####  unit convirsion for bandwidth Threshold Byte to MB #####################################
        config['bandwidth_threshold'].field_dict['value'] = eplglobals.fromByte(config['bandwidth_threshold'].field_dict['value'],'MB')
        ### creating dict extra to make default selection in kid pages ###
        extra = {'siteinfo_security':security_level,'ipinfo_namebased':namebased,'plan_name':args['plan_name']}
        
        template.config = config
        template.extra = extra

        return self.kidserialize (template)
        
    editPlan.exposed = True

    def doeditPlan(self,**formargs):
        
        ###### used convertStringToList() method to convert string with \n in list format for alias ##
        if 'aliases_aliases' in formargs:
            formargs['aliases_aliases'] = parsealias.convertStringToList(formargs['aliases_aliases'],'\n')
        ############# create map for Security Level ###################################################
        security_level = {'high':{'apache_jail':'1','telnet_jail':'1','ssh_jail':'1'},'medium':{'apache_jail':'0',\
            'telnet_jail':'1','ssh_jail':'1'},'low':{'apache_jail':'0','telnet_jail':'0','ssh_jail':'0'}}
        formargs.update(security_level[formargs['siteinfo_security']])
        #####  unit convirsion for bandwidth Threshold MB to Byte #####################################
        if 'bandwidth_threshold' in formargs:
            formargs['bandwidth_threshold'] = eplglobals.toByte(formargs['bandwidth_threshold'],'MB')
       
        ############################ Calling Backend for edit Plan  ################################# 
        try:
            plan = eplapiCurrentUser().plans[formargs['plan_name']].editPlan(**formargs)
        except EplError:
            error(_('Unable to edit Plan'))
        return self.listplans()
        
    doeditPlan.exposed = True

    def dodeletePlan(self,**args):
        try:
            plan = eplapiCurrentUser().deletePlan(args['plan_name'])
        except EplError:
            error(_('Unable to delete Plan'))
        return self.listplans()

    dodeletePlan.exposed = True

    def _rewrite_svc(self,config,**formargs):
        for svc in config:
            key = config[svc].field_dict['name']
            if key in formargs:
                if config[svc].field_type == 'Bool':
                    config[svc].field_dict['checked'] = bool(int(formargs[key]))
                elif config[svc].field_type == 'ListVal':
                    config[svc].field_dict['value'] = parsealias.convertStringToList(formargs[key],'\n')
                elif config[svc].field_type == 'Select':
                    for opt in config[svc].options:
                        if opt.field_dict['value'] == formargs[key]:
                            opt.field_dict['selected'] = 'True'
                else:
                    config[svc].field_dict['value']= formargs[key]
            
        return config
    
    _rewrite_svc.exposed = True

    def _complete_domainnames(self,config,domain):
	for fieldname in config:
	   if fieldname == 'aliases_aliases':
		domain_aliases = []
		aliases = config[fieldname].field_dict['value']
		domainprefix = '.'.join(domain.split('.')[:-1])
		for alias in aliases:
		    merge_alias = domainprefix + alias
		    if merge_alias != domain and alias.startswith('.'):
			domain_aliases.append(merge_alias)
		config[fieldname].field_dict['value'] = domain_aliases
	   elif fieldname.find('server') != -1:
		field_value = config[fieldname].field_dict['value']
		if field_value.endswith('.'):
		   config[fieldname].field_dict['value'] = config[fieldname].field_dict['value'] + domain

    _complete_domainnames.exposed = True

    def editSite (self,**args):
        """
        This function provides the edit site page.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.editsite)
        # create the config that needs to be sent to the kid pages
        config = dict()
        
        # get the edit site tmpl containing the field properties to display. 
        if len(args)>1 and 'editsite' not in args:
            serverobj = eplapi.server()
            plan_filter = {'ipinfo_namebased':args['ipinfo_namebased']}
            addsitetmpl = serverobj.createSiteSignature( plan_name=args['plan_name'], **plan_filter)
        else:
            siteobj = eplapi.sites(args['site_name'])
            addsitetmpl = siteobj.editSignature()
            
        for key, obj in addsitetmpl.hashed_fields.items():
            config[key] = addsitetmpl.hashed_fields[key]
            
        if len(args)>1 and 'editsite' in args:
            config = self._rewrite_svc(config,**args)
            if 'bandwidth_threshold' in args:
                config['bandwidth_threshold'].field_dict['value'] = args['bandwidth_threshold']
        elif len(args)>1 and 'editsite' not in args:
	    ##### complete domain for ftp,mail,web and aliases ###
	    self._complete_domainnames(config,args['siteinfo_domain'])
		
            config['siteinfo_domain'].field_dict['value'] = args['siteinfo_domain']
            config['siteinfo_admin_user'].field_dict['value'] = args['siteinfo_admin_user']
            config['siteinfo_passwd1'].field_dict['value'] = args['siteinfo_passwd1']
            config['siteinfo_passwd2'].field_dict['value'] = args['siteinfo_passwd2']
            config['siteinfo_email'].field_dict['value'] = args['siteinfo_email']
            config['ipinfo_ipaddrs'].field_dict['value'] = args['ipinfo_ipaddrs_txt']
            config['bandwidth_threshold'].field_dict['value'] = eplglobals.fromByte(\
                    config['bandwidth_threshold'].field_dict['value'],'MB')
            config['ipinfo_namebased'].field_dict['checked'] = bool(int(args['ipinfo_namebased']))
            config['mysql_dbaseadmin'].field_dict['value'] = 'admin'
            if args['siteinfo_domain']!='':
                domainstr = args['siteinfo_domain']
                config['mysql_dbaseprefix'].field_dict['value'] = "_".join(domainstr.split(".")) + "_-_"

            apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
            telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
            ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
            if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                    security_level = 'high'
            elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                    security_level = 'medium'
            elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                    security_level = 'low'
            args['siteinfo_security'] = security_level
        else :
            apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
            telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
            ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
            if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                    security_level = 'high'
            elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                    security_level = 'medium'
            elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                    security_level = 'low'
            args.update({'siteinfo_security':security_level})
            config['bandwidth_threshold'].field_dict['value'] = eplglobals.fromByte(config['bandwidth_threshold'].field_dict['value'],'MB')
            
        config['ipinfo_nbaddrs'].field_dict['value']= eplapi.services('apache').getNameVirtualHost()[0]
        config['aliases_aliases'].field_dict['constraints'] = '[\S]*$'
        extra= {'ipinfo_namebased': str(int(bool(config['ipinfo_namebased'].field_dict['checked']))),'siteinfo_security':args['siteinfo_security'],'site_name':args['site_name'],'is_pro':eplglobals.isPro()}

        template.config = config
        template.extra = extra
        
        return self.kidserialize (template)
        
    editSite.exposed = True

    def doeditSite (self,**formargs):
       
        old_formargs = dict(formargs)
        
        siteobj = eplapi.sites(formargs['site_name'])
        ########################### check Ipbasesd site or namebased site here #######################
	formargs['ipinfo_nbaddrs'] = [formargs['ipinfo_nbaddrs']]
        if formargs['ipinfo_namebased'] == '0':
            formargs['ipinfo_ipaddrs'] = [formargs['ipinfo_ipaddrs']]
            del(formargs['ipinfo_nbaddrs'])
            
        ###### used convertStringToList() method to convert string with \n in list format for alias ##
        if 'aliases_aliases' in formargs:
            formargs['aliases_aliases'] = parsealias.convertStringToList(formargs['aliases_aliases'],'\n')
            
        ############# create map for Security Level ###################################################
        security_level = {'high':{'apache_jail':'1','telnet_jail':'1','ssh_jail':'1'},'medium':{'apache_jail':'0',\
        'telnet_jail':'1','ssh_jail':'1'},'low':{'apache_jail':'0','telnet_jail':'0','ssh_jail':'0'}}
        formargs.update(security_level[formargs['siteinfo_security']])
            
        #####  unit convirsion for bandwidth Threshold MB to Byte #####################################
        if 'bandwidth_threshold' in formargs:
            formargs['bandwidth_threshold'] = eplglobals.toByte(formargs['bandwidth_threshold'],'MB')
            
        if formargs['plan_name']=='none':
            formargs['plan_name']='default'

        try:
            siteobj.edit(**formargs)
        except EplError:
            error(_("Edit Site failed"))
            return self.editSite(**old_formargs)
        return self.listsites()
        #################### Calling Backend method to adds site ########################################
        #actionlogid = lib.threads.keepalive.keepalive (siteobj.edit,formargs)
        #self.kidglobals (kidpages.isp.liveactionlog)
        #kidpages.isp.liveactionlog.id = actionlogid
        #return self.kidserialize (kidpages.isp.liveactionlog)

    doeditSite.exposed = True
    
    def addIpSite (self , **args):
        args.update({'ipinfo_namebased':'0'})
        return self.addNbSite(**args)
        
    addIpSite.exposed = True

    def doaddIpsite (self, **formargs):
        return self.doaddNbsite(**formargs)
        
    doaddIpsite.exposed = True

    def addIpsiteadv (self, **formargs):
        return self.addNbsiteadv(**formargs)

    addIpsiteadv.exposed = True
    
    def addNbSite (self,**args):
        """
        This function provides the basic add site page.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.addsite)
        
        # create the config that needs to be sent to the kid pages
        config = dict()
        serverobj = eplapi.server()
        plandicts = serverobj.user.plans
        # get the add site tmpl containing the field properties to display. 
        if 'ipinfo_namebased' not in args:
            args.update({'ipinfo_namebased':'1'})
        if 'plan_name' not in args:
            args.update({'plan_name':'default'})
            
        extra = {'ipinfo_namebased':args['ipinfo_namebased'],'method_type':'simple','plandicts':plandicts,'site_name':'','is_pro': eplglobals.isPro()}
        
        addsitetmpl = serverobj.createSiteSignature(args['plan_name'],**extra)
        
        for key, obj in addsitetmpl.hashed_fields.items():
            config[key] = addsitetmpl.hashed_fields[key]
        
        ### selecting security level acording to plan, when control come from list plan
        apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
        telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
        ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
        if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                security_level = 'high'
        elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                security_level = 'medium'
        elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                security_level = 'low'
        extra.update({'siteinfo_security':security_level})
        
        config['siteinfo_domain'].field_dict['value'] = ''
        config['siteinfo_admin_user'].field_dict['value'] = ''
        config['siteinfo_passwd1'].field_dict['value'] = ''
        config['siteinfo_passwd2'].field_dict['value'] = ''
        config['siteinfo_email'].field_dict['value'] = ''
        
        template.config = config
        template.extra = extra

        return self.kidserialize (template)
        
    addNbSite.exposed = True
    
    def doaddNbsite (self, **formargs):
        """
        This function makes the call to add site.
        """
        import eplapi.commonlib.fields
        serverobj = eplapi.server()
        # get the add site tmpl containing the field properties for the selected plan. 
        addsitetmpl = serverobj.createSiteSignature( plan_name=formargs['plan_name'])

        # do the final call to add the site.
        # The control reaches here under 2 cases:
        # 1. The user clicks on button "Add Site" on the Basic add site page.
        # 2. The user clicks on button "Add Site" on the Advance add site page.
        # TODO: add this as a hidden field
        formargs['ipinfo_nbaddrs'] = eplapi.services('apache').getNameVirtualHost()[0]
	formargs['ipinfo_nbaddrs'] = [formargs['ipinfo_nbaddrs']]
        old_formargs = {}
        old_formargs.update(formargs)
           
        ########################### check Ipbasesd site or namebased site here #######################
        if formargs['ipinfo_namebased'] == '0':
            formargs['ipinfo_ipaddrs'] = [formargs['ipinfo_ipaddrs']]
            del(formargs['ipinfo_nbaddrs'])
            
        ###### used convertStringToList() method to convert string with \n in list format for alias ##
        if 'aliases_aliases' in formargs:
            formargs['aliases_aliases'] = parsealias.convertStringToList(formargs['aliases_aliases'],'\n')
            
        ############# create map for Security Level ###################################################
        security_level = {'high':{'apache_jail':'1','telnet_jail':'1','ssh_jail':'1'},'medium':{'apache_jail':'0',\
            'telnet_jail':'1','ssh_jail':'1'},'low':{'apache_jail':'0','telnet_jail':'0','ssh_jail':'0'}}
        formargs.update(security_level[formargs['siteinfo_security']])
            
        #####  unit convirsion for bandwidth Threshold MB to Byte #####################################
        if 'bandwidth_threshold' in formargs:
            formargs['bandwidth_threshold'] = eplglobals.toByte(formargs['bandwidth_threshold'],'MB')
          
        ############### removed unused key from formargs ##############################################
        plan_name = formargs['plan_name']
        del formargs['plan_name']
            
        try:
            #action_id = keepalive(serverobj.createSite, (plan_name,),formargs) # keepalive
            serverobj.createSite(plan_name,**formargs)
        except EplError:
            error(_("Create Site failed"))
            if old_formargs['ipinfo_namebased']=='1':
                return self.addNbsiteadv(**old_formargs)
            if old_formargs['ipinfo_namebased']=='0':
                return self.addIpsiteadv(**old_formargs)
        except Exception, e:
            log_exception()

        #template = self.kidglobals (kidpages.common.liveactionlog) # keepalive
        #template.action_id = action_id # keepalive
        #return self.kidserialize (template) # keepalive
        return self.listsites()
        
    doaddNbsite.exposed = True

    
    def addNbsiteadv (self, **formargs):
        import eplapi.commonlib.fields
        serverobj = eplapi.server()
        # get the add site tmpl containing the field properties for the selected plan. 
        addsitetmpl = serverobj.createSiteSignature( plan_name=formargs['plan_name'])

        # get the advance page of the site.
        # get the add site tmpl containing the field properties to display. 
        extra= {'ipinfo_namebased':formargs['ipinfo_namebased'],'siteinfo_security':formargs['siteinfo_security']\
                ,'method_type':'advanced','site_name':'','is_pro': eplglobals.isPro()}
            
        addsitetmpl = serverobj.createSiteSignature( plan_name=formargs['plan_name'],**extra)
        config = addsitetmpl.hashed_fields
            
        #####  unit convirsion for bandwidth Threshold Byte to MB #####################################
        config['bandwidth_threshold'].field_dict['value'] = eplglobals.fromByte(config['bandwidth_threshold'].field_dict['value'],'MB')
            
        debug ('form length == %s' % len(formargs))
        # restoring new value for siteinfo service and 
        if 'addsite' in formargs and len(formargs)>20:
            config = self._rewrite_svc(config,**formargs)
            if 'bandwidth_threshold' in formargs :
                config['bandwidth_threshold'].field_dict['value'] = formargs['bandwidth_threshold']
            
        # added FQDN for ftp,mail,web
        if len(formargs)<20 or (len(formargs)>20 and 'addsite' not in formargs) :
	    ##### complete domain for ftp,mail,web and aliases ###
	    self._complete_domainnames(config,formargs['siteinfo_domain'])
	
            config['siteinfo_domain'].field_dict['value'] = formargs['siteinfo_domain']
            config['siteinfo_admin_user'].field_dict['value'] = formargs['siteinfo_admin_user']
            config['siteinfo_passwd1'].field_dict['value'] = formargs['siteinfo_passwd1']
            config['siteinfo_passwd2'].field_dict['value'] = formargs['siteinfo_passwd2']
            config['siteinfo_email'].field_dict['value'] = formargs['siteinfo_email']
            config['ipinfo_namebased'].field_dict['value'] = formargs['ipinfo_namebased']
            if formargs['ipinfo_namebased'] == '0':
                config['ipinfo_ipaddrs'].field_dict['value'] = formargs['ipinfo_ipaddrs']
            config['mysql_dbaseadmin'].field_dict['value'] = formargs['siteinfo_admin_user']
            if formargs['siteinfo_domain']!='':
                str = formargs['siteinfo_domain']
                config['mysql_dbaseprefix'].field_dict['value'] = "_".join(str.split(".")) + "_-_"

            apache_jail = int(bool(config['apache_jail'].field_dict['checked']))
            telnet_jail = int(bool(config['telnet_jail'].field_dict['checked']))
            ssh_jail = int(bool(config['ssh_jail'].field_dict['checked']))
            if apache_jail == 1 and  telnet_jail == 1 and ssh_jail == 1:
                    security_level = 'high'
            elif telnet_jail == 1 and ssh_jail == 1 and apache_jail == 0:
                    security_level = 'medium'
            elif telnet_jail == 0 and ssh_jail == 0 and apache_jail == 0:
                    security_level = 'low'
    
            if len(formargs)<20 :
                extra['siteinfo_security'] = formargs['siteinfo_security']
            else:
                extra['siteinfo_security'] = security_level
        ########################### assigning Alias default constraints ##################
        config['aliases_aliases'].field_dict['constraints'] = '[\S]*$'

        #assigning template instance to variable 'template'
        template = self.kidglobals ( kidpages.isp.addsiteadv )
        template.config = config
        template.extra = extra
            
        return self.kidserialize (template)

    addNbsiteadv.exposed = True

    def suspendResume(self,**args):
        try:
            serverobj = eplapi.server()
            if args['is_resumed']=='1':
                serverobj.suspendSite(args['site_name'])
            elif args['is_resumed']=='2':
                serverobj.resumeSite(args['site_name'])
        except:
            error(_("operation has been failed"))

        return self.listsites()
            
    suspendResume.exposed = True
    
    #####################################Server configuration panel###########################################

    def networksettings(self):
        #assigning template instance to variable 'template' 
        template = self.kidglobals (kidpages.isp.networksettings)
        serverobj = eplapi.server()
        networksettingsdict = serverobj.getNetworkSettings()
        template.config = networksettingsdict
        return self.kidserialize (template)

    networksettings.exposed = True

    def uploadlogo(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.common.uploadlogo)
        serverobj = eplapi.server()
        template.config = serverobj.uploadLogoSignature().hashed_fields
        return self.kidserialize (template)

    uploadlogo.exposed = True

    def douploadlogo(self, myFile=None, upload=None, remove=None):
        serverobj = eplapi.server()
        path = '/etc/appliance/customization/logo.gif'
        try:
            if upload:
                serverobj.uploadLogo(logo=myFile, path=path)
                info(_('Upload Logo Successfull'))
            elif remove:
                serverobj.removeLogo(logo=path)
                info(_('Reset Logo to Default Logo Successfull'))
        except Exception, err:
            error(_(err))
        return self.uploadlogo()

    douploadlogo.exposed = True
    
    def logo_gif(self):
        logo_path = '/etc/appliance/customization/logo.gif'
        if not os.path.exists(logo_path):
            logo_path = os.path.join(SKINROOT, cherrypy.session['skin'], 'images/logo.gif')
        body = serveFile(logo_path)
        try: del cherrypy.response.headerMap ['Last-Modified']
        except KeyError: pass
        return body

    logo_gif.exposed = True

    def manageskins(self):
        template = self.kidglobals (kidpages.common.manageskins)
        template.config = eplapiCurrentUser().manageskinsSignature().hashed_fields
        return self.kidserialize (template)
        
    manageskins.exposed = True

    def domanageskins(self, apply=None, preview=None, skin=None):
        if apply:
            eplapiCurrentUser().manageskins(skin=skin)
            info(_('Skin changed to %s') % skin)
        elif preview:
            template = self.kidglobals(kidpages.common.previewskin)
            template.image = '/static/images/%s_preview.gif' % skin
            return self.kidserialize(template)
        return self.manageskins()
        
    domanageskins.exposed = True

    def diskusage(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.diskusage)
        serverobj = eplapi.server()
        diskusagelist = serverobj.getDiskInfo()
        template.config = diskusagelist
        return self.kidserialize (template)

    diskusage.exposed = True

    def timezone(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.timezone)
        serverobj = eplapi.server()
        time = serverobj.getTime()
	list=time.split()
	time=reduce(lambda string, item: string + ' ' + _(item), list, "").strip()
        config = {'time': time}
        template.config = config
        return self.kidserialize (template)

    timezone.exposed = True
    
    def edittimezone(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.edittimezone)
        serverobj = eplapi.server()
        config = serverobj.setTimeZoneSignature().hashed_fields
        template.config = config
        return self.kidserialize (template)

    edittimezone.exposed = True

    def settimezone(self, timezone):
        serverobj = eplapi.server()
        serverobj.setTimeZone(timezone=timezone)
        return self.timezone()

    settimezone.exposed = True

    def show_serveraction(self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.show_serveraction)
        config = {}
        template.config = config
        return self.kidserialize (template)
        
    show_serveraction.exposed = True

    def serveraction(self, restart=None, reboot=None):
        serverobj = eplapi.server()

        if restart:
            info(_('You are now logged out.The Control Panel has been scheduled to restart in 1 minute. Please wait a few minutes before reconnecting.'))
            serverobj.restart()
            
        elif reboot:
            info(_('You are now logged out. The server has been scheduled to reboot in 1 minute. Please wait a few minutes before reconnecting.'))
            serverobj.reboot()

        return '''<html><script>window.open("/welcome","_top")</script></html>'''


    serveraction.exposed = True

    def autologinsite (self,siteadm):
        """ auto-logs in to a site. `siteadm' is the name of the site administrator """
        siteadmin = eplapi.roles ('siteadmin') (siteadm)
        # If we reached here, it means that it's a legit site admin user

        # try to establish where we want to return back after our login session finishes. we can try to get this
        # from the referer
        #try: returnurl = cherrypy.request.headerMap ['Referer']
        #except: returnurl = "/isp/"
        autologinflag = True

        self.pushtologinsstack (siteadmin, autologinflag)
        url = "/%s/" % siteadmin.role
        raise cherrypy.HTTPRedirect(url, 302)

    autologinsite.exposed = True

    def changeadmin (self):
        #assigning template instance to variable 'template'
        template = self.kidglobals(kidpages.isp.changeadmin)

        editobj = eplapiCurrentUser().editSignature()
        template.config = editobj.hashed_fields
        return self.kidserialize (template)

    changeadmin.exposed = True

    def dochangeadmin (self, AdminName, AdminEmail, passwd, passwd2, save=None, cancel=None):
        try:
            eplapiCurrentUser().edit(AdminName=AdminName, passwd=passwd, passwd2=passwd2, AdminEmail=AdminEmail)
            info(_("Server Administrator updated successfully."))
        except EplError :
            pass
        return self.changeadmin()
    dochangeadmin.exposed = True
    
    def changelicense (self):
        #assigning template instance to variable template

        config = dict()
        user = eplapiCurrentUser()
        editobj = eplapiCurrentUser().changeMLKSignature()
        for key, obj in editobj.hashed_fields.items():
            config[key] = editobj.hashed_fields[key]
        template = self.kidglobals(kidpages.isp.changelicense)
        template.config = config
        template.level = 0
        template.fromPage = 'license'
        return self.kidserialize (template)

    changelicense.exposed = True

    def displayLicense(self):
        #assigning template instance to variable template
        template = self.kidglobals( kidpages.isp.displayLicense )
        try:
            lic_data = licenselib.getLicense()
            lic_dict = EncryptDecrypt.decryptLicense(lic_data)
            template.license = lic_dict
        except EplError:
            template.license = None
        return self.kidserialize (template)
        
    displayLicense.exposed = True

    def dochangelicense (self, **conf):
        try:
            if 'part1' not in conf:
                return self.renderChangeLic('index')
            if 'hidflag' in conf and conf['hidflag'] == '1':
                cherrypy.session['skipforcelicense'] = 1
                if 'trespasser' in cherrypy.session:
                    del cherrypy.session['trespasser']
		template = self.kidglobals( kidpages.isp.index )
                return self.kidserialize (template)
            MLK = conf['part1'] + conf['part2'] + conf['part3'] + conf['part4'] + conf['part5']
            if MLK =='' or not EncryptDecrypt.validateMLK(MLK) or licenselib.isProToBasicMLK(MLK):
                if MLK =='' or not EncryptDecrypt.validateMLK(MLK):
                    error(_("Invalid License Key."))
                elif licenselib.isProToBasicMLK(MLK):
                    self.displayError("PRO_TO_BASIC_NOT_ALLOWED")

                return self.renderChangeLic(conf['fromPage'])
            oldMLK = licenselib.getMLK()
            if oldMLK != 0:
                oldconf = {'part1':oldMLK[:5],'part2':oldMLK[5:10],'part3':oldMLK[10:15],'part4':oldMLK[15:20],'part5':oldMLK[20:]}
            if MLK != oldMLK:
                eplapiCurrentUser().changeMLK(**conf)
                (returnCode,returnMsg) = callhome.callMe(1)
                if returnCode == 2:
                    if oldMLK != 0:
                        eplapiCurrentUser().changeMLK(**oldconf)
                    else:
                        eplapiCurrentUser().deleteMLK()
                    actionlog.clean()
                    self.displayError(returnMsg)
                    return self.renderChangeLic(conf['fromPage'])
                else:
                    basic_to_pro = 0
                    (info2, capacity2, gracedays2) = EncryptDecrypt.getMLKInfo(MLK)
                    if oldMLK:
                        (info1, capacity1, gracedays1) = EncryptDecrypt.getMLKInfo(oldMLK)
                        if info1['basic'] and not info2['basic']:
                            basic_to_pro = 1
                            info(_("License Key is changed from Basic to Pro. Restart epld for changes to take effect."))
                    initialstate = SystemState()
                    initialstate.updateState(initialstate)
                    if info2['basic']:
                       initialstate['product_type'] = 'basic' 
                    else:
                       initialstate['product_type'] = 'pro' 
                    initialstate.basic_to_pro = basic_to_pro
                    try:
                        initialstate.commit()
                        obj_factory.events.OnHideUnHide()
                    except:
                        pass
                    info(_("License Key is changed."))
            else:
                eplapiCurrentUser().changeMLK(**conf)
                (returnCode,returnMsg) = callhome.callMe()
                if returnCode == 2:
                    self.displayError(returnMsg)
                    return self.renderChangeLic(conf['fromPage'])
                else:
                    basic_to_pro = 0
                    (info2, capacity2, gracedays2) = EncryptDecrypt.getMLKInfo(MLK)
                    initialstate = SystemState()
                    initialstate.updateState(initialstate)
                    if info2['basic']:
                       initialstate['product_type'] = 'basic' 
                    else:
                       initialstate['product_type'] = 'pro' 
                    initialstate.basic_to_pro = basic_to_pro
                    try:
                        initialstate.commit()
                        obj_factory.events.OnHideUnHide()
                    except:
                        pass
        except EplError :
            error(_("Error Updating License Key."))
	if conf['fromPage'] == 'index':
            if licenselib.isGrace() or licenselib.isExpired():
                return self.renderChangeLic(conf['fromPage'])
            else:
                if 'trespasser' in cherrypy.session:
                   del cherrypy.session['trespasser']
                template = self.kidglobals( kidpages.isp.index )
                return self.kidserialize (template)
	else:
            return self.renderChangeLic(conf['fromPage'])

    dochangelicense.exposed = True

    def seteula(self, **args):

        eplapi.server().seteula()
        if not licenselib.getLicense():
            return self.renderChangeLic('index')
        else:
            if 'trespasser' in cherrypy.session:
               del cherrypy.session['trespasser']
            template = self.kidglobals( kidpages.isp.index )
            return self.kidserialize (template)

    seteula.exposed = True

    def renderChangeLic(self,frompage):
    
        try:
            MLK = licenselib.getMLK()
            if MLK:
                (product,capacity,gracePeriod) = EncryptDecrypt.getMLKInfo()
                graceDaysLapsed = licenselib.effectiveGraceDays()
            else:
                gracePeriod = 0
                graceDaysLapsed = 0
	except Exception, ex:
            pass

        config = dict()
        user = eplapiCurrentUser()
        editobj = eplapiCurrentUser().changeMLKSignature()
        for key, obj in editobj.hashed_fields.items():
            config[key] = editobj.hashed_fields[key]
        template = self.kidglobals (kidpages.isp.changelicense)
        template.config = config
        skip =1
        if (MLK == 0 or frompage == 'license') and skip:
            template.level = 0
            skip = 0
            
        if licenselib.isGrace() and skip:
            if gracePeriod - graceDaysLapsed > 0:
                template.level = 1
                error(_('The system failed to register product for the following reasons:'))
                error(_('1.Unable to contact the licensing server to validate the license.'))
                error(_('2.Improper system\'s Date/Time settings.'))
                error(_('You can register the product by entering a valid License Key and clicking \"Register\" button.'))
                error(_('To use Ensim Control Panel during the evaluation period click \"Continue\". Ensim Control Panel will continue to work for the duration of the evaluation period (%s days). Register Ensim Control Panel during this period, otherwise you will be unable to use it once the evaluation period expires.') , (str(gracePeriod - graceDaysLapsed)))
            else:
                template.level = 2 
                error(_("Grace period of %s days is over.") , str(gracePeriod))
            skip = 0
        elif licenselib.isExpired() and skip:
            template.level = 2 
            if licenselib.isExpired() == 2:
                error(_("License is expired. Provide a valid License Key."))
            skip = 0
        else:
            template.level = 2 

        template.fromPage = frompage
        return self.kidserialize (template)

    def displayError(self,returnMsg):
        if returnMsg == "MSGID_ELS_REG_NOAVAILABLELICENSES":
            error(_("Invalid License Key."))
            error(_('License is freezed or no instance available. Provide a valid License Key.'))
        elif returnMsg == "MSGID_ELS_REG_INVALIDIP":
            error(_('Given IP is blocked.'))
        elif returnMsg == "DOMAIN_LIMIT_EXCEEDED":
            error(_("Invalid License Key."))
            error(_('Domain limit for this License Key is less than no. of domains present. Provide a valid License Key.'))
        elif returnMsg == "PRO_TO_BASIC_NOT_ALLOWED":
            error(_("Invalid License Key."))
            error(_('Ensim Basic License Key cannot be applied to a Ensim Pro system. Provide a valid License Key.'))
        elif returnMsg == "SERVER_NOT_CONTACTABLE":
            error(_('Server is not currently contactable.'))
        else:
            error(_("Invalid License Key."))
            error(_("License is blocked. Provide a valid License Key."))

    def clearusersession (self):
        """ clears all the user specific variables from the session """
        print 'isp clear session'
        for item in ['username', 'role', 'helprole', 'userguiroot', 'skin', 'dashboard', 'subnavbar']:
            if item in cherrypy.session: del cherrypy.session[item]

