import cherrypy
import rootapp.pages
from cherrypy.lib import httptools

# import the epl interface
import eplapi
import wpidn

import kidpages.common.generallist
import kidpages.isp.services.bind.soasettings
import kidpages.isp.services.bind.editsoa
import kidpages.isp.services.bind.addzone
import kidpages.isp.services.bind.edit_primarynameserver
import kidpages.isp.services.bind.edit_secnameserver
import kidpages.isp.services.bind.edit_virtualnameserver
import kidpages.isp.services.bind.addedit_A_record
import kidpages.isp.services.bind.addedit_NS_record
import kidpages.isp.services.bind.addedit_MX_record
import kidpages.isp.services.bind.addedit_PTR_record
import kidpages.isp.services.bind.addedit_CNAME_record
import kidpages.isp.services.bind.enableDNS

from lib.datamodel.controls import Column, ActionLink
from lib.datamodel.ispviews import NameServerListView
from lib.datamodel.ispviews import ZoneListView
from lib.datamodel.ispviews import RecordListView

svc_name = 'bind'

class Page (rootapp.pages.ISPPage):
    def index (self):
        return "<html>Implementation in progress</html>"

    index.exposed = True

    def displaysoasettings(self, **kwargs):
        """
        This function displays the SOA settings of the EPL Server or the zone provided as the keyword argument.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.soasettings)
        bind_sc = eplapi.services(svc_name)

        if kwargs.has_key('zonename'):
            zoneobj = bind_sc.getZone(kwargs['zonename'])
            config = zoneobj.getSOASettings()
        else:
            config = bind_sc.getDefaultSOASettings()

        template.config = config
        return self.kidserialize (template)

    displaysoasettings.exposed = True


    def editsoasettings(self, **kwargs):
        """
        This function shows the current SOA settings of the EPL Server or zone. 
        These settings can be edited by the user.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.editsoa)
        bind_sc = eplapi.services(svc_name)
        config = dict()

        if kwargs.has_key('zonename'):
            # get the soa of the zone
            zoneobj = bind_sc.getZone(kwargs['zonename'])
            soatmpl = zoneobj.setZoneSOASettingsSignature()
        else:
            # get the soa settings of the server
            soatmpl = bind_sc.setDefaultSOASettingsSignature()

        for key, obj in soatmpl.hashed_fields.items():
            config[key] = soatmpl.hashed_fields[key]

        template.config = config
        if kwargs.has_key('zonename'):
            template.zone = kwargs['zonename']
        else:
            template.zone = None
        return self.kidserialize (template)

    editsoasettings.exposed = True

    def doeditsoasettings(self, **formargs):
        """
        This function calls the eplapi function to edit the SOA settings of the EPL Server
        or the zone. The settings of the zone are edited if the zone name is provided in the
        form arguement, else the EPL server settings are edited. 
        """
        bind_sc = eplapi.services(svc_name)

        if 'zonename' in formargs:
            # set the soa of the zone
            zoneobj = bind_sc.getZone(formargs['zonename'])
            try:
                zoneobj.setZoneSOASettings(formargs['master'],formargs['email'],formargs['refresh'],formargs['retry'],formargs['expire'],formargs['ttl'])
                info (_("SOA Settings updated successfully."))
            except:
                logexception()
            # return to the view soa settings page of the zone
            return self.displaysoasettings(zonename=formargs['zonename'])

        else:
            try:
                # set the soa settings of the server
                bind_sc.setDefaultSOASettings(formargs['email'],formargs['refresh'], formargs['retry'],formargs['expire'],formargs['ttl'])
                info (_("Default SOA Setting  updated successfully."))
            except EplError:
                logexception()
            # return to the view soa settings page of the server
            return self.displaysoasettings()

    doeditsoasettings.exposed = True


    def displaynameservers(self):
        """
        This function displays the Name Servers.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals ( kidpages.common.generallist )
        bind_sc = eplapi.services(svc_name)

        nameserverlist = []
        
        # get the primary server details
        nameserver = bind_sc.getMasterServer()
        # create a dict for fliplist entry for primary server.
        config = {}
        config['type'] = _("Primary Master")
        config['servername'] = nameserver['primary_master_server']
        config['actions'] = ['edit']
        config['servertype'] = "primary"
        nameserverlist.append(config)

        # get the secondary server
        nameserver = bind_sc.getSlaveList()
        # create a dict for fliplist entry for secondary server.
        config = {}
        config['type'] = _("Secondary Server")

        config['actions'] = ['edit']
        if nameserver:
            config['servername'] = nameserver
            config['actions'].append('delete')
        else:
            config['servername'] = ""
        config['servertype'] = "secondary"
        nameserverlist.append(config)

        # get the virtual dns server
        nameserver = bind_sc.getVirtualDNS()
        # create a dict for fliplist entry for virtual dns server.
        config = {}
        config['type'] = _("Virtual DNS Server")
        config['actions'] = ['edit']
        if nameserver:
            config['servername'] = nameserver['virt_dns_server']
            config['serverip'] = nameserver['virt_dns_ip']
            config['actions'].append('delete')
        else:
            config['servername'] = ""
            config['serverip'] = "" 
        config['servertype'] = "virtual"
        nameserverlist.append(config)
 
        # provide mysitelist to the generallist kid page.
        column_list = [
            Column (name = 'type',        title = _('Name server Type')),
            Column (name = 'servername',  title = _('Server name'), searchable = True, sortable = True),
            Column (name = 'actions',     title = _('Actions'))
        ]

        template.buttons = None
        template.page_title = _('Name Server List')
        template.list_title = _('Name Server List')
        template.column_list = column_list
        template.display_data = nameserverlist
        template.entryview = NameServerListView
        template.display_view = "NAMESERVERLIST"

        return self.kidserialize (template)

    displaynameservers.exposed = True


    def editprimarynameserver(self):
        """
        This function shows the page to edit the Primary Name Server.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.edit_primarynameserver)
        bind_sc = eplapi.services(svc_name)
        primarytmpl = bind_sc.setMasterServerSignature()
        template.config = primarytmpl.hashed_fields
        return self.kidserialize (template)

    editprimarynameserver.exposed = True

    def doeditprimarynameserver(self, **formargs):
        """
        This function calls the eplapi function to edit the the Primary Name Server.
        """
        bind_sc = eplapi.services(svc_name)
        try:
            bind_sc.setMasterServer(formargs['primary_master_server'])
            info (_("Primary Master updated"))
        except EplError, e:
            pass
        return self.displaynameservers()

    doeditprimarynameserver.exposed =  True


    def editsecnameserver(self):
        """
        This function shows the page to edit the Secondary Name Server.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.edit_secnameserver)
        bind_sc = eplapi.services(svc_name)
        sectmpl = bind_sc.editSlaveServerSignature()
        template.config = sectmpl.hashed_fields
        return self.kidserialize (template)

    editsecnameserver.exposed = True

    def doeditsecnameserver(self, **formargs):
        """
        This function calls the eplapi function to edit the the Secondary Name Server.
        """
        bind_sc = eplapi.services(svc_name)
        try:
            if formargs['oldservername'] == "":
                # do add secondary name server
                bind_sc.addSlaveServer(formargs['servername'])
                # delete virtual_dns server if present
                nameserver = bind_sc.getVirtualDNS()
                if nameserver!=[]:
                    bind_sc.removeVirtualDns(nameserver['virt_dns_server'])
                    info (_("Virtual DNS removed"))
            else:
                # do edit secondary name server
                bind_sc.editSlaveServer(formargs['oldservername'], formargs['servername'])
                
            info (_("Secondary Server updated"))
        except EplError, e:
            pass
        
        return self.displaynameservers()

    doeditsecnameserver.exposed =  True


    def editvirtualnameserver(self):
        """
        This function shows the page to edit the Virtual Name Server.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.edit_virtualnameserver)
        bind_sc = eplapi.services(svc_name)
        vtmpl = bind_sc.editVirtualDnsSignature()
        template.config = vtmpl.hashed_fields
        return self.kidserialize (template)

    editvirtualnameserver.exposed = True


    def doeditvirtualnameserver(self, **formargs):
        """
        This function calls the eplapi function to edit theVirtual Name Server settings.
        """
        bind_sc = eplapi.services(svc_name)
        if formargs['old_virt_dns_server'] == "" and formargs['old_virt_dns_ip'] == "":
            # add the virtual dns server
            bind_sc.addVirtualDns(formargs['virt_dns_server'],formargs['virt_dns_ip'])
            # delete the secondary server if present
            nameserver = bind_sc.getSlaveList()
            if nameserver!=[]:
                bind_sc.removeSlaveServer(nameserver[0])
                info (_("Secondary Server removed"))
        else:
           #edit the virtual dns server
           bind_sc.editVirtualDns(formargs['virt_dns_server'],formargs['virt_dns_ip'])

        info (_("Virtual DNS Added"))

        return self.displaynameservers()

    doeditvirtualnameserver.exposed =  True

    def deletenameservers(self, **formargs):
        """
        This functions calls the eplapi function to delete the name servers. The types 
        of name servers that can be deleted are secondary servers and virtual dns servers.
        """ 
        bind_sc = eplapi.services(svc_name)

        if formargs['type'] == "secondary":
            bind_sc.removeSlaveServer(formargs['servername'])
            info (_("Secondary Server removed"))

        if formargs['type'] == "virtual":
            bind_sc.removeVirtualDns(formargs['servername'])
            info (_("Virtual DNS removed"))

        return self.displaynameservers()
    deletenameservers.exposed =  True

    # START Zone related operations
    def displayzonelist(self):
        """
        This function displays the list of zones.
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals ( kidpages.common.generallist )
        bind_sc = eplapi.services(svc_name)

        dispzonelist = []
        
        # get the zone list
        allzonelist=bind_sc.getZoneList() 
        # create a dict for fliplist entries.
        for each_zone in allzonelist:
            config = {}
            config['zonename'] = each_zone.zone
            config['actions'] = ['viewsoa','viewrec','delete']
            dispzonelist.append(config)

        # provide mysitelist to the generallist kid page.
        column_list = [
            Column (name = 'zonename',  title = _('Zone name'), searchable = True, sortable = True),
            Column (name = 'actions',     title = _('Actions'))
        ]

        template.buttons = None
        template.page_title = _('Zone List')
        template.list_title = _('Zone List')
        template.column_list = column_list
        template.display_data = dispzonelist
        template.entryview = ZoneListView
        template.display_view = "ZONELIST"
        template.actionLinks = [ActionLink (display = _("Add Zone"), href = '/isp/services/bind/addzone')]

        return self.kidserialize (template)

    displayzonelist.exposed = True


    def addzone(self, **formargs):
        """
        This function publishes the add zone form. 
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addzone)
        bind_sc = eplapi.services(svc_name)
        addzonetmpl = bind_sc.addZoneSignature()
        template.config = addzonetmpl.hashed_fields
        return self.kidserialize (template)

    addzone.exposed = True

   
    def doaddzone(self, **formargs):
        """
        This function adds the zone. 
        """
        try:
            bind_sc = eplapi.services(svc_name)
            bind_sc.addZone(formargs['zonename'])
            info (_("Zone added successfully."))
        except EplError, e:
            pass

        # take the user back to the zone list page.
        return self.displayzonelist()

    doaddzone.exposed = True


    def deletezone(self, zonename):
        """
        This function deletes the zone. The zone name must be provided as the arguement to this function. 
        """
        try:
            bind_sc = eplapi.services(svc_name)
            bind_sc.removeZone(zonename)
            info (_("Zone removed successfully."))
        except EplError,err:
            pass

        # take the user back to the zone list page.
        return self.displayzonelist()

    deletezone.exposed = True
    # END Zone related operations

    # START Record related operations
    def displayrecordlist(self, zonename):
        """
        This function displays the DNS records for the zone.
        """                
        #assigning template instance to variable 'template'
        template = self.kidglobals ( kidpages.common.generallist )
        bind_sc = eplapi.services(svc_name)

        disprecordlist = []
        
        # get the record list for the selected zone
        zoneinfo = bind_sc.getZoneInfo(zonename)
        allrecordlist = zoneinfo['record_list'] 

        # create a dict for fliplist entries.
        for each_rec in allrecordlist:
            config = {}
            config['hostname'] = each_rec.list_name()
            config['type'] = each_rec.rec_type
            config['data'] = each_rec.list_data()
            config['actions'] = ['edit','delete']

            # extra fields required for edit/delete operations. 
            # save minimum information to create the record object for edit/delete operations. 
            config['zonename'] = each_rec.zone
            if each_rec.rec_type == 'A':
                config['ipaddr'] = each_rec.ipaddr
                config['short_hostname'] = each_rec.short_hostname
            if each_rec.rec_type == 'NS':
                config['ns_server'] = each_rec.ns_server
                config['short_ns_zone'] = each_rec.short_ns_zone
                config['ipaddr'] = ""
            if each_rec.rec_type == 'MX':
                config['ipaddr'] = ""
                config['short_mxdomain'] = each_rec.short_mxdomain
                config['mxserver'] = each_rec.mxserver
                config['precedence'] = each_rec.precedence
            if each_rec.rec_type == 'PTR':
                config['ipaddr'] = each_rec.ipaddr
            if each_rec.rec_type == 'CNAME':
                config['alias'] = each_rec.alias
                config['target'] = each_rec.target
            disprecordlist.append(config)

        # provide mysitelist to the generallist kid page.
        column_list = [
            Column (name = 'hostname',  title = _('Host'), searchable = True, sortable = True),
            Column (name = 'type',  title = _('Record Type'), searchable = True, sortable = True),
            Column (name = 'data',  title = _('Data'), searchable = True, sortable = True),
            Column (name = 'actions',     title = _('Actions'))
        ]

        template.buttons = None
        template.page_title = _('Record List')
        template.list_title = _('Record List for domain ') + wpidn.punycode_to_utf8(zoneinfo['domain'])
        template.column_list = column_list
        template.display_data = disprecordlist
        template.entryview = RecordListView
        template.display_view = "RECORDLIST"
        
        if zonename.endswith("in-addr.arpa"):
            template.actionLinks = \
                [
                ActionLink (display = _("Add NS Record"), href='add_records?zonename=%s&type=NS' % zonename),
                ActionLink (display = _("Add PTR Record"), href='add_records?zonename=%s&type=PTR' % zonename)
                ]
        else:
            template.actionLinks = \
                [
                ActionLink (display = _("Add A Record"), href='add_records?zonename=%s&type=A' % zonename),
                ActionLink (display = _("Add NS Record"), href='add_records?zonename=%s&type=NS' % zonename),
                ActionLink (display = _("Add CNAME Record"), href='add_records?zonename=%s&type=CNAME' % zonename),
                ActionLink (display = _("Add MX Record"), href='add_records?zonename=%s&type=MX' % zonename)
                ]

        return self.kidserialize (template)

    displayrecordlist.exposed = True

    def deleterecord(self, **kwargs):
        """
        This function deletes the record from the zone. The record to be deleted is
        identified by the kwyword arguements provieded to the function.
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])

        rec_type = kwargs['type']
        try:
            if rec_type == 'A':
                zoneobj.removeARecord(kwargs['hostname'],kwargs['ipaddr'])
            elif rec_type == 'NS':
                zoneobj.removeNSRecord(kwargs['hostname'],kwargs['ns_server'])
            elif rec_type == 'MX':
                zoneobj.removeMXRecord(kwargs['hostname'],kwargs['mxserver'],kwargs['precedence'])
            elif rec_type == 'PTR':
                zoneobj.removePTRRecord(kwargs['hostname'],kwargs['ipaddr'])
            elif rec_type == 'CNAME':
                zoneobj.removeCNAMERecord(kwargs['alias'],kwargs['target'])
            info (_("Record removed successfully."))
        except EplError,e:
            pass

        # delete complete, return to the record list for the zone.
        return self.displayrecordlist(kwargs['zonename'])

    deleterecord.exposed = True

    def add_records(self, **kwargs):
        """
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
      
        rec_type = kwargs['type']
        if rec_type == 'A':
            recordtmpl = zoneobj.addARecordSignature()
            record_kid = kidpages.isp.services.bind.addedit_A_record
        if rec_type == 'NS':
            recordtmpl = zoneobj.addNSRecordSignature()
            record_kid = kidpages.isp.services.bind.addedit_NS_record
        if rec_type == 'MX':
            recordtmpl = zoneobj.addMXRecordSignature()
            record_kid = kidpages.isp.services.bind.addedit_MX_record
        if rec_type == 'PTR':
            recordtmpl = zoneobj.addPTRRecordSignature()
            record_kid = kidpages.isp.services.bind.addedit_PTR_record
        if rec_type == 'CNAME':
            recordtmpl = zoneobj.addCNAMERecordSignature()
            record_kid = kidpages.isp.services.bind.addedit_CNAME_record

        #assigning template instance to variable 'template'
        template = self.kidglobals (record_kid)
        template.config = recordtmpl.hashed_fields
        template.zone = kwargs['zonename']
        return self.kidserialize (template)

    add_records.exposed = True


    def edit_A_record(self, **kwargs):
        """
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addedit_A_record)
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
        recordtmpl = zoneobj.editARecordSignature(kwargs['short_hostname'], kwargs['ipaddr'])
        template.config = recordtmpl.hashed_fields 
        template.zone = kwargs['zonename'] 
        return self.kidserialize (template)

    edit_A_record.exposed = True

    def edit_NS_record(self, **kwargs):
        """
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addedit_NS_record)
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
        recordtmpl = zoneobj.editNSRecordSignature(kwargs['short_ns_zone'], kwargs['ns_server'])
        template.config = recordtmpl.hashed_fields 
        template.zone = kwargs['zonename'] 
        return self.kidserialize (template)

    edit_NS_record.exposed = True

    def edit_MX_record(self, **kwargs):
        """
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addedit_MX_record)
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
        recordtmpl = zoneobj.editMXRecordSignature(kwargs['short_mxdomain'], kwargs['mxserver'], kwargs['precedence'])
        template.config = recordtmpl.hashed_fields 
        template.zone = kwargs['zonename'] 
        return self.kidserialize (template)

    edit_MX_record.exposed = True


    def edit_PTR_record(self, **kwargs):
        """
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addedit_PTR_record)
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
        recordtmpl = zoneobj.editPTRRecordSignature(kwargs['ipaddr'], kwargs['hostname'])
        template.config = recordtmpl.hashed_fields 
        template.zone = kwargs['zonename'] 
        return self.kidserialize (template)

    edit_PTR_record.exposed = True


    def edit_CNAME_record(self, **kwargs):
        """
        """
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.services.bind.addedit_CNAME_record)
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(kwargs['zonename'])
        recordtmpl = zoneobj.editCNAMERecordSignature(kwargs['alias'], kwargs['target'])
        template.config = recordtmpl.hashed_fields 
        template.zone= kwargs['zonename'] 
        return self.kidserialize (template)

    edit_CNAME_record.exposed = True

    def doaddeditArecord(self, **formargs):
        """
        This functions calls the eplapi function to add/edit the A record. 'Add' or 'edit
        operation is identified by the parameters passed to this function. If the keyword
        argument 'formargs' contains old information of the Address record, the function to
        be called is identified as edit, else it is add.  
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(formargs['zonename'])
        # check box issue, explicitly setting the value
        if formargs.has_key('rec_rev_update'):
            rec_rev_update = 1
        else:
            rec_rev_update = 0

        try:
            if formargs.has_key('old_short_hostname') and formargs.has_key('old_ipaddr'):
                zoneobj.editARecord(formargs['old_short_hostname'],formargs['old_ipaddr'], formargs['short_hostname'],formargs['ipaddr'], rec_rev_update)
                info (_("Record updated successfully."))
            else:
                zoneobj.addARecord(formargs['short_hostname'],formargs['ipaddr'], rec_rev_update)
                info (_("Record successfully added"))
        except EplError,e:
            logexception()

        # return to the record list page
        return self.displayrecordlist(formargs['zonename'])

    doaddeditArecord.exposed = True


    def doaddeditCNAMErecord(self, **formargs):
        """
        This functions calls the eplapi function to add/edit the CNAME record. 'Add' or 'edit
        operation is identified by the parameters passed to this function. If the keyword
        argument 'formargs' contains old information of the record, the function to
        be called is identified as edit, else it is add.  
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(formargs['zonename'])

        try:
            if formargs.has_key('old_alias') and formargs.has_key('old_target'):
                zoneobj.editCNAMERecord(formargs['old_alias'],formargs['old_target'], formargs['alias'],formargs['target'])
                info (_("Record updated successfully."))
            else:
                zoneobj.addCNAMERecord(formargs['alias'],formargs['target'])
                info (_("Record successfully added"))
        except EplError,e:
            logexception()
        
        # return to the record list page
        return self.displayrecordlist(formargs['zonename'])

    doaddeditCNAMErecord.exposed = True


    def doaddeditNSrecord(self, **formargs):
        """
        This functions calls the eplapi function to add/edit the NS record. 'Add' or 'edit
        operation is identified by the parameters passed to this function. If the keyword
        argument 'formargs' contains old information of the record, the function to
        be called is identified as edit, else it is add.  
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(formargs['zonename'])

        try:
            if formargs.has_key('old_short_ns_zone') and formargs.has_key('old_ns_server'):
                zoneobj.editNSRecord(formargs['old_short_ns_zone'],formargs['old_ns_server'], formargs['short_ns_zone'],formargs['ns_server'])
                info (_("Record updated successfully."))
            else:
                zoneobj.addNSRecord(formargs['short_ns_zone'],formargs['ns_server'])
                info (_("Record successfully added"))
        except EplError,e:
            logexception()

        # return to the record list page
        return self.displayrecordlist(formargs['zonename'])

    doaddeditNSrecord.exposed = True


    def doaddeditMXrecord(self, **formargs):
        """
        This functions calls the eplapi function to add/edit the MX record. 'Add' or 'edit
        operation is identified by the parameters passed to this function. If the keyword
        argument 'formargs' contains old information of the record, the function to
        be called is identified as edit, else it is add.  
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(formargs['zonename'])

        try:
            if formargs.has_key('old_short_mxdomain') and formargs.has_key('old_mxserver') and  formargs.has_key('old_precedence'):
                zoneobj.editMXRecord(formargs['old_short_mxdomain'],formargs['old_mxserver'], formargs['old_precedence'], formargs['short_mxdomain'], formargs['mxserver'], formargs['precedence'])
                info (_("Record updated successfully."))
            else:
                zoneobj.addMXRecord(formargs['short_mxdomain'], formargs['mxserver'], formargs['precedence'])
                info (_("Record successfully added"))
        except EplError,e:
            logexception()

        # return to the record list page
        return self.displayrecordlist(formargs['zonename'])

    doaddeditMXrecord.exposed = True


    def doaddeditPTRrecord(self, **formargs):
        """
        This functions calls the eplapi function to add/edit the PTR record. 'Add' or 'edit
        operation is identified by the parameters passed to this function. If the keyword
        argument 'formargs' contains old information of the Address record, the function to
        be called is identified as edit, else it is add.  
        """
        bind_sc = eplapi.services(svc_name)
        zoneobj = bind_sc.getZone(formargs['zonename'])
        # check box issue, explicitly setting the value
        if formargs.has_key('update_forward'):
            update_forward = 1
        else:
            update_forward = 0
        try:
            if formargs.has_key('old_ipaddr') and formargs.has_key('old_hostname'):
                arguments = {'old_ipaddr':formargs['old_ipaddr'], 'old_hostname':formargs['old_hostname'], 'ipaddr':formargs['ipaddr'], 'hostname':formargs['hostname'], 'update_forward':update_forward}
                zoneobj.editPTRRecord(**arguments)
                info (_("Record updated successfully."))
            else:
                zoneobj.addPTRRecord(formargs['hostname'], formargs['ipaddr'], update_forward)
                info (_("Record successfully added"))
        except EplError,e:
            logexception()
            
        # return to the record list page
        return self.displayrecordlist(formargs['zonename'])

    doaddeditPTRrecord.exposed = True

    def enableDnsOnSites(self):
        required_sitelist = []
        
        serverobj = eplapi.server()
        siteobjlist = serverobj.getAllSiteObjs()
        for siteobj in siteobjlist:
            if siteobj.state!=1 and siteobj.state!=3: # if site is in incosistent state
                continue                             # do not include it in the list.
            if 'bind' not in siteobj.enabled_services:
                required_sitelist.append(siteobj.domain)
                
        template = self.kidglobals (kidpages.isp.services.bind.enableDNS)
        template.required_sitelist = required_sitelist
        return self.kidserialize (template)
        
    enableDnsOnSites.exposed = True

    def doenableDnsOnSites (self, **formargs):
        serverobj = eplapi.server()
        
        sites_checked = []
        for site, checked in formargs.items():
            if checked=='on':
                sites_checked.append(site)
                
        siteobjlist = serverobj.getAllSiteObjs()
        try:
            for siteobj in siteobjlist:
                if siteobj.domain in sites_checked:
                    editsitetmpl = siteobj.editSignature()
                
                    config = {}
                    config['siteinfo_domain'] = editsitetmpl.hashed_fields['siteinfo_domain'].field_dict['value']
                    config['siteinfo_admin_user'] = editsitetmpl.hashed_fields['siteinfo_admin_user'].field_dict['value']
                    config['siteinfo_passwd1'] = editsitetmpl.hashed_fields['siteinfo_passwd1'].field_dict['value']
                    config['siteinfo_passwd2'] = editsitetmpl.hashed_fields['siteinfo_passwd2'].field_dict['value']
                    config['siteinfo_email'] = editsitetmpl.hashed_fields['siteinfo_email'].field_dict['value']
                    config['bind_enabled'] = '1'
                
                    siteobj.edit(**config)
        except EplError, err:
            pass
    
        return self.enableDnsOnSites()

    doenableDnsOnSites.exposed = True

    # END Record related operations

    def main (self):
        #assigning template instance to variable 'template'
        template = self.kidglobals (kidpages.isp.dashboard)
        return self.kidserialize (template)

    main.exposed = True

