import rootapp.pages

import eplapi

import kidpages.common.subdomain
import kidpages.site.services.subdomain.info
import kidpages.site.services.subdomain.defaults
import kidpages.site.services.subdomain.editdefaults
import kidpages.site.services.subdomain.addsubdomain
import kidpages.site.services.subdomain.editsubdomain

from lib.datamodel.controls import Column
from lib.datamodel.siteviews import SubDomainListEntryView

class Page (rootapp.pages.SitePage):

    def showlist (self):
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.common.subdomain)

        obj = eplapi.services('subdomain')
        objlist = obj.list()

        domainlist = []

        for domainobj in objlist:
            s = obj.subdomains(domainobj)
            config = {}
            config['name'] = domainobj
            config['owner'] = s.info['owner']
            if s.info['cgi'] == '1':
                config['cgi'] = 'Enabled'
            else:
                config['cgi'] = 'Disabled'
            config['actions'] = 'none'

            domainlist.append(config)

        column_list = [
                Column (name = 'name', title = _('Name'), searchable = True, sortable = True),
                Column (name = 'owner', title = _('Owner'), searchable = True, sortable = True),
                Column (name = 'cgi', title = _('CGI'), searchable = True, sortable = True),
                Column (name = 'actions', title = _('Actions'))
            ]

        template.list_title = _('Subdomain List')
        template.column_list = column_list
        template.display_data = domainlist
        template.entryview = SubDomainListEntryView
        template.display_view = _("SUBDOMAINLIST")

        return self.kidserialize (template)
    showlist.exposed = True
    
    def showsubdomain (self, name):
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.site.services.subdomain.info)
        obj = eplapi.services('subdomain')
        s = obj.subdomains(name)
        config = dict(name=name, owner=s.info['owner'], path=s.info['document_root'], url=s.info['access_url'],
                      cgidir = s.info['cgi_root'])
        if s.info['cgi'] == '1':
            config['cgi'] = _('Yes')
        else:
            config['cgi'] = _('No')
        template.config = config
        return self.kidserialize (template)

    showsubdomain.exposed = True

    def defaults (self):
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.site.services.subdomain.defaults)
        obj = eplapi.services('subdomain')
        config = {}
        config['cgi'] = obj.defaults['cgi']
        config['base'] = obj.defaults['base']
        config['owner'] = obj.defaults['owner']
        template.config = config
        return self.kidserialize (template)

    defaults.exposed = True

    def editsubdomain (self, name):
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.site.services.subdomain.editsubdomain)
        serviceobj = eplapi.services('subdomain')

        # get the edit subdomain tmpl containing the field properties to display.
        editsubdomaintmpl = serviceobj.editSignature(name)
        template.config =  editsubdomaintmpl.hashed_fields
        template.ownerlist = [option.field_dict['Label'] for option in editsubdomaintmpl.hashed_fields['owner'].options]

        s = serviceobj.subdomains(name)
        oldconfig = s.info
        if s.info['cgi']=='1':
            oldconfig['cgi']='Enabled'
        else:
            oldconfig['cgi']='Disabled'

        template.oldconfig = oldconfig
        return self.kidserialize (template)
    editsubdomain.exposed = True

    def doeditsubdomain (self, **conf):
        try:
            serviceobj = eplapi.services('subdomain')
            serviceobj.edit(**conf)
        except EplError:
            pass
        return self.showlist()

    doeditsubdomain.exposed = True
                                        
    def deletesubdomain (self, name):
        obj = eplapi.services('subdomain')
        obj.delete(name)
            
        return self.showlist()

    deletesubdomain.exposed = True

    def editdefaults (self, **query):
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.site.services.subdomain.editdefaults)
        config = eplapi.services('subdomain').defaultsSignature().hashed_fields
        template.config = config
        return self.kidserialize (template)

    editdefaults.exposed = True

    def doeditdefaults (self, **conf):
        obj = eplapi.services('subdomain')
        obj.defaults = dict (owner = conf['owner'], cgi = conf['cgi'])
        return self.defaults()

    doeditdefaults.exposed = True

    def addsubdomain (self):
        
        serviceobj = eplapi.services('subdomain')
        maxSubdomainCount = int(serviceobj.site.getServiceConfig()['subdomain']['max'])
        currentSubdomainCount = len(serviceobj.list())
        if currentSubdomainCount >= maxSubdomainCount:
            error(_('Not more than %d subdomain(s) are allowed for this site' % maxSubdomainCount))
            return self.showlist()
      
        #assigning tempalte instance to variable 'template'
        template = self.kidglobals (kidpages.site.services.subdomain.addsubdomain)
        # get the add subdomain tmpl containing the field properties to display.
        template.config = serviceobj.addSignature().hashed_fields
        template.ownerlist = [option.field_dict['Label'] for option in template.config['owner'].options]
        template.defaultvals = serviceobj.defaultsSignature().hashed_fields
        template.basedir = serviceobj.defaults['base']
        return self.kidserialize (template)

    addsubdomain.exposed = True

    def doaddsubdomain (self, name, owner, document_root, cgi='0'):
        serviceobj = eplapi.services('subdomain')
        
        try:
            serviceobj.add(name, owner, serviceobj.defaults['base'] + '/' + document_root, cgi)
        except EplError:
            return self.addsubdomain()
        return self.showlist()

    doaddsubdomain.exposed = True
